Package: vandalico
Type: Package
Title: Evaluation of Presence-Absence Models
Version: 0.2.2
Date: 2026-04-14
Authors@R: c(person(given = "Alberto", family = "Jiménez-Valverde",
           role = c("aut", "cre"),
           email = "alberto.jimenez.valverde@gmail.com",
           comment = c(ORCID = "0000-0001-9962-2106")))
Maintainer: Alberto Jiménez-Valverde <alberto.jimenez.valverde@gmail.com>
Description: Collection of functions to evaluate presence-absence models. It 
    comprises functions to adjust discrimination statistics for the 
    representativeness effect through case-weighting, along with functions for 
    visualizing the outcomes. Originally outlined in: 
    Jiménez-Valverde (2022) The uniform AUC: dealing with the 
    representativeness effect in presence-absence models. Methods Ecol. Evol, 
    13, 1224-1236.
Acknowledgments: This work was supported by the Ministerio de Ciencia,
        Innovación y Universidades/Agencia Estatal de
        Investigación/FEDER (PID2023-147672NB-I00), and by CSIC
        (2024ICT253).
License: GPL-3
Imports: ROCR (>= 1.0-7), stats, graphics
Depends: R (>= 4.3.0)
RoxygenNote: 7.3.3
Encoding: UTF-8
NeedsCompilation: no
Packaged: 2026-04-14 08:38:27 UTC; User
Author: Alberto Jiménez-Valverde [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-9962-2106>)
Repository: CRAN
Date/Publication: 2026-04-14 10:20:02 UTC
Built: R 4.4.3; ; 2026-04-14 12:13:42 UTC; unix

Package: vandalico
Type: Package
Title: Evaluation of Presence-Absence Models
Version: 0.2.2
Date: 2026-04-14
Authors@R: c(person(given = "Alberto", family = "Jiménez-Valverde",
           role = c("aut", "cre"),
           email = "alberto.jimenez.valverde@gmail.com",
           comment = c(ORCID = "0000-0001-9962-2106")))
Maintainer: Alberto Jiménez-Valverde <alberto.jimenez.valverde@gmail.com>
Description: Collection of functions to evaluate presence-absence models. It 
    comprises functions to adjust discrimination statistics for the 
    representativeness effect through case-weighting, along with functions for 
    visualizing the outcomes. Originally outlined in: 
    Jiménez-Valverde (2022) The uniform AUC: dealing with the 
    representativeness effect in presence-absence models. Methods Ecol. Evol, 
    13, 1224-1236.
Acknowledgments: This work was supported by the Ministerio de Ciencia,
        Innovación y Universidades/Agencia Estatal de
        Investigación/FEDER (PID2023-147672NB-I00), and by CSIC
        (2024ICT253).
License: GPL-3
Imports: ROCR (>= 1.0-7), stats, graphics
Depends: R (>= 4.3.0)
RoxygenNote: 7.3.3
Encoding: UTF-8
NeedsCompilation: no
Packaged: 2026-04-14 08:38:27 UTC; User
Author: Alberto Jiménez-Valverde [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-9962-2106>)
Repository: CRAN
Date/Publication: 2026-04-14 10:20:02 UTC
Built: R 4.5.2; ; 2026-04-15 05:00:52 UTC; unix

# Generated by roxygen2: do not edit by hand

export(AUCuniform)
export(AUCuniform.2)
export(AUCuniform_trap)
export(CALplot)
export(HSgraph)
importFrom(grDevices,rgb)
importFrom(graphics,abline)
importFrom(graphics,axis)
importFrom(graphics,hist)
importFrom(graphics,lines)
importFrom(graphics,plot)
importFrom(graphics,points)
importFrom(stats,quantile)
importFrom(stats,wilcox.test)
importFrom(utils,head)
importFrom(utils,tail)

#' @title Calibration graph
#'
#' @description A function to plot a calibration graph.
#' @param mat A matrix with two columns. The first column must contain the
#' suitability values (i.e., the classification rule); the second column must
#' contain the presences and absences.
#' @param by Size of the suitability intervals (bins). By default,
#' \code{by} = 0.1.
#' @details Dots for bins with 15 or more cases are shown in solid black; dots
#' for bins with less than 15 cases are shown empty (see Jiménez-Valverde et
#' al. 2013). This way, by plotting the calibration graph before running
#' \code{\link{AUCuniform}}, one can get a glimpse of how reliable \emph{uAUC}
#' or \emph{uSe*} can be expected to be.
#' @return This function returns a calibration plot
#' @examples
#' suit<-rbeta(100, 2, 2) # Generate suitability values
#' random<-runif(100)
#' sp<-ifelse(random < suit,1 , 0) # Generate presence-absence data
#' CALplot(cbind(suit, sp))
#' @encoding UTF-8
#' @references Jiménez-Valverde, A., Acevedo, P., Barbosa, A. M., Lobo, J. M. &
#'     Real, R. (2013).  Discrimination capacity in species distribution models
#'     depends on the representativeness of the environmental domain.
#'     \emph{Global Ecol. Biogeogr.}, 22, 508-516.
#' @export

CALplot <- function(mat, by = 0.1) {
    bins <- cut(mat[, 1], seq(0, 1, by), include.lowest = T, right = F)
    tableBins <- table(mat[, 2], bins)
    prevalBins <- as.matrix(tableBins[2, ]/colSums(tableBins))
    prevalBins.2 <- as.matrix(tapply(mat[, 1], bins, mean))
    colDots <- ifelse(colSums(tableBins) < 14, 21, 19)
    plot(prevalBins.2, prevalBins, main = "Calibration plot", pch = colDots, ylab = "observed probability", xlab = "predicted probability",
        xlim = c(0, 1), ylim = c(0, 1), yaxt = "n", cex.lab = 1.3, cex = 1.1)
    axis(side = 2, las = 2, mgp = c(3, 0.75, 0))
    abline(a = 0, b = 1, col = "black", lty = 2)
}

#' @title Calculates the uniform \emph{AUC} and uniform \emph{Se*} by the
#' trapezoidal method.
#'
#' @description This function computes the uniform \emph{AUC} (\emph{uAUC}) and
#' uniform \emph{Se*} (\emph{uSe*}) using the weighted trapezoidal method
#' instead of the weighted bootstrapping method used in \code{AUCuniform} and
#' originally proposed in Jiménez-Valverde (2022). This procedure reduces bias 
#' and improves the coverage of confidence intervals (Jiménez-Valverde 2024). 
#' Additionally, the weights vector associated to each case can be customized. 
#' See Jiménez-Valverde (2024) for details.
#' @param mat A matrix with two columns. The first column must contain the
#' suitability values (i.e., the classification rule); the second column must
#' contain the presences and absences.
#' @param by Size of the suitability intervals (i.e., bins). By default,
#' \code{by} = 0.1.
#' @param deleteBins A vector (e.g., from 1 to 10 if \code{by} = 0.1) with the
#' bins that have to be excluded (1 for [0,0.1), 10 for [0.9,1]) from the
#' calculation of the uniform statistics; \code{NULL} by default.
#' @param w A vector with the weights associated to each case. If \code{NULL}
#' (default), then the uniform \emph{AUC} (\emph{uAUC}) and uniform \emph{Se*}
#' (\emph{uSe*}) are calculated.
#' @param plot Logical. Indicates whether or not the observed ROC curve is
#' plotted (gray dots).
#' @param plot.compare Logical. Indicates whether or not the weighed ROC curve
#' is plotted (black line).
#' @param plot.adds Logical. Indicates whether or not the negative diagonal and
#' the points of equivalence (weighted and unweighted) are added to the ROC
#' plot.
#' @details This function calculates the uniform \emph{AUC} (\emph{uAUC}) and
#' uniform \emph{Se*} (\emph{uSe*}) using the weighted trapezoidal method as
#' suggested in Jiménez-Valverde (2024). A warning message will be shown if
#' the sample size of any bin is zero. Another warning message will be shown if
#' the sample size of any bin is lower than 15. In such case, trimming should be
#' considered using \code{deleteBins} (Jiménez-Valverde 2022). Alternatively,
#' the weights associated to each case can be fully customized with the \code{w}
#' parameter (Jiménez-Valverde 2024). In this case, no warnings regarding
#' sample size issues are raised, and \code{deleteBins} is not used. The
#' \emph{AUC} (non-uniform, unweighted) is estimated non-parametrically by the
#' trapezoidal rule, which is equivalent to the Wilcoxon-based estimation 
#' (Hanley & McNeil 1982) used in \code{AUCuniform}. \emph{Se*} is calculated as 
#' in \code{AUCuniform}.
#' @return A list with the following elements:
#' @return \code{AUC}: the \emph{AUC} value (non-uniform, unweighted), a numeric
#' value between 0 and 1.
#' @return \code{Se}: the \emph{Se*} value (non-uniform, unweighted), a numeric
#' value between 0 and 1.
#' @return \code{bins}: a table with the sample size of each bin (only if
#' \code{w = NULL}).
#' @return \code{uAUC}: the uniform \emph{AUC} value (only if \code{w = NULL}).
#' @return \code{uSe}: the uniform \emph{Se*} value (only if \code{w = NULL}).
#' @return \code{wAUC}: the weighted \emph{AUC} estimated with the vector
#' \code{w}
#' @return \code{wSe}: the weighted \emph{Se*} estimated with the vector
#' \code{w}
#' @examples
#' \donttest{
#' suit<-rbeta(100, 2, 2) #Generate suitability values
#' random<-runif(100)
#' sp<-ifelse(random < suit, 1, 0) #Generate presence-absence data
#' result<-AUCuniform_trap(cbind(suit, sp), plot = TRUE, plot.compare = TRUE)
#' result$AUC #Get the AUC
#' result$uAUC #Get the uAUC. Note how it is closer to the reference value of
#'             #0.83 since the suitability values are simulated to be
#'             #well-calibrated (see Jimenez-Valverde 2022).
#'             }
#' @encoding UTF-8
#' @references Hanley, J. A. & McNeil, B. J. (1982). The Meaning and Use of the
#'     Area under a Receiver Operating Characteristic (ROC) Curve.
#'     \emph{Radiology}., 143, 29-36.
#'
#'     Jiménez-Valverde, A. (2022). The uniform AUC: dealing with the
#'     representativeness effect in presence-absence models. \emph{Methods Ecol.
#'     Evol.}, 13, 1224-1236.
#'
#'     Jiménez-Valverde, A. (2024). Improving the uniform AUC (uAUC): towards a 
#'     case-by-case weighting evaluation in species distribution models. 
#'     \emph{In preparation}. 
#' @importFrom graphics plot
#' @importFrom graphics abline
#' @importFrom graphics axis
#' @importFrom graphics points
#' @keywords internal
#' @export

AUCuniform_trap <- function(mat, by = 0.1, deleteBins = NULL, w = NULL, plot = FALSE, plot.compare = FALSE, plot.adds = FALSE) {
    .Deprecated("AUCuniform.2")
    # Non-uniform:
    mat.ord <- mat[order(mat[, 1], decreasing = TRUE), ]
    true.pos <- cumsum(mat.ord[, 2])/sum(mat.ord[, 2])
    false.pos <- cumsum(1 - mat.ord[, 2])/sum((1 - mat.ord[, 2]))
    auc.trap <- sum(diff(c(0, false.pos)) * true.pos)
    diferencia <- abs(true.pos - (1 - false.pos))
    se.trap <- (true.pos[which.min(diferencia)] + (1 - false.pos)[which.min(diferencia)])/2
    if (plot == TRUE) {
        plot(c(0, false.pos), c(0, true.pos), pch = 16, xlab = "false positive rate", ylab = "sensitivity", main = "ROC curve",
            yaxt = "n", cex.lab = 1.3, cex.axis = 1, col = "gray")
        axis(side = 2, las = 2, mgp = c(3, 0.75, 0))
        abline(a = 0, b = 1, lty = 2)
    }
    # Uniform:
    if (is.null(w) == TRUE) {
        bins <- seq(0, 1, by)
        intervals <- cut(mat[, 1], bins, include.lowest = TRUE, right = FALSE)
        probs <- as.vector(1/table(intervals)[intervals])
        probs.ord <- probs[order(mat[, 1], decreasing = TRUE)]
        if (dim(mat)[1] < 30)
            warning("Your sample size is low, results must be interpreted with caution.")
        if (sum(table(intervals) < 15) > 0)
            warning("At least one sutability interval has n < 15, results must be interpreted with caution.")
        if (sum(table(intervals) == 0) > 0)
            warning(paste("There are", sum(table(intervals) == 0), "interval(s) with zero data, results must be interpreted with caution."))
        if (is.null(deleteBins) == FALSE) {
            toDelete <- levels(intervals)[deleteBins]
            mat <- mat[(intervals %in% toDelete) == FALSE, ]
            probs <- probs[(intervals %in% toDelete) == FALSE]
            mat.ord <- mat[order(mat[, 1], decreasing = TRUE), ]
            probs.ord <- probs[order(mat[, 1], decreasing = TRUE)]
        }
        wauc <- sum(diff(c(0, cumsum((1 - mat.ord[, 2]) * probs.ord)/sum((1 - mat.ord[, 2]) * probs.ord))) * (cumsum(mat.ord[,
            2] * probs.ord)/sum(mat.ord[, 2] * probs.ord)))
        diferencia <- abs((cumsum(mat.ord[, 2] * probs.ord)/sum(mat.ord[, 2] * probs.ord)) - (1 - (cumsum((1 -
            mat.ord[, 2]) * probs.ord)/sum((1 - mat.ord[, 2]) * probs.ord))))
        wse <- ((cumsum(mat.ord[, 2] * probs.ord)/sum(mat.ord[, 2] * probs.ord))[which.min(diferencia)] + (1 -
            (cumsum((1 - mat.ord[, 2]) * probs.ord)/sum((1 - mat.ord[, 2]) * probs.ord))))[which.min(diferencia)]/2
    }
    # Other weights for cases
    if (is.null(w) == FALSE) {
        if (length(w) != dim(mat)[1])
            warning("The number of cases does not match the length of w. The weighted statistics are not calculated and the weighted ROC curve is not plotted.")
        probs.ord <- w[order(mat[, 1], decreasing = TRUE)]
        wauc <- sum(diff(c(0, cumsum((1 - mat.ord[, 2]) * probs.ord)/sum((1 - mat.ord[, 2]) * probs.ord))) * (cumsum(mat.ord[,
            2] * probs.ord)/sum(mat.ord[, 2] * probs.ord)))
        wse <- ((cumsum(mat.ord[, 2] * probs.ord)/sum(mat.ord[, 2] * probs.ord))[which.min(diferencia)] + (1 -
            (cumsum((1 - mat.ord[, 2]) * probs.ord)/sum((1 - mat.ord[, 2]) * probs.ord))))[which.min(diferencia)]/2
    }
    # Plots
    if (plot.compare == TRUE) {
        lines(c(0, (cumsum((1 - mat.ord[, 2]) * probs.ord)/sum((1 - mat.ord[, 2]) * probs.ord))), c(0, (cumsum(mat.ord[,
            2] * probs.ord)/sum(mat.ord[, 2] * probs.ord))), pch = 16, col = "black")
    }
    if (plot.adds == TRUE) {
        abline(a = 1, b = -1, col = "darkgrey", lty = 2)
        points(1 - se.trap, se.trap, col = "red", pch = 16)
        points(1 - wse, wse, col = "red", pch = 16)
    }
    # Get results
    if (is.null(w) == TRUE) {
        return(list(AUC = auc.trap, Se = se.trap, bins = table(intervals), uAUC = wauc, uSe = wse))
    }
    if (is.null(w) == FALSE) {
        return(list(AUC = auc.trap, Se = se.trap, wAUC = wauc, wSe = wse))
    }
}

#' @title Calculates the uniform \emph{AUC} and uniform \emph{Se*} by the
#' direct weighted trapezoidal estimation method.
#'
#' @description This function computes the uniform \emph{AUC} (\emph{uAUC}) and
#' uniform \emph{Se*} (\emph{uSe*}) using the direct weighted trapezoidal
#' estimation method (Jiménez-Valverde 2025), instead of the stratified 
#' bootstrapping with inverse probability weighting method implemented in 
#' \code{\link{AUCuniform}} and originally proposed by Jiménez-Valverde (2022). 
#' Uniform statistics are design to account for the representativeness effect 
#' (Jiménez-Valverde 2022). This new method reduces bias and improves the 
#' coverage of confidence intervals relative to the original proposal. 
#' Additionally, the weight vector associated to each case can be customized. 
#' @param mat A matrix with two columns. The first column must contain the 
#' classification rule (e.g., the suitability values); the second column 
#' must contain the presences and absences.
#' @param by The size of the intervals used to divide the classification rule 
#' (i.e., bins width). By default, \code{by} = 0.1. This argument is only used 
#' when \code{w = NULL}.
#' @param deleteBins A vector (e.g., from 1 to 10 if \code{by} = 0.1) with the
#' bins that have to be excluded (1 for [0,0.1), 10 for [0.9,1]) from the
#' calculation of the uniform statistics. The default is \code{NULL}. This 
#' argument is only used when \code{w = NULL}. 
#' @param w A vector with the weights associated with each case. If \code{NULL}
#' (default), each case is weighted by the inverse of the sample size of 
#' its corresponding bin, and the uniform \emph{AUC} (\emph{uAUC}) and uniform 
#' \emph{Se*} (\emph{uSe*}) are calculated (see Details).
#' @param plot Logical. If \code{TRUE}, the standard (unweighted) ROC curve is 
#' plotted (gray dots).
#' @param plot.compare Logical. If \code{TRUE}, the weighed ROC curve is plotted
#' (black line).
#' @param plot.adds Logical. If \code{TRUE}, adds the negative diagonal and the
#' points of equivalence (weighted and unweighted) to the ROC plot.
#' @details This function calculates the uniform \emph{AUC} (\emph{uAUC}) and
#' uniform \emph{Se*} (\emph{uSe*}) using the direct weighted trapezoidal 
#' estimation method proposed by Jiménez-Valverde (2025). To compute the uniform
#' statistics, the \emph{w} parameter must be set to \emph{NULL} (default). The 
#' data set is divided into bins (defined by the parameter \code{by}) based on 
#' the values of the first vector in the input matrix \code{mat} (the 
#' classification rule). Each observation is assigned a weight equal to one 
#' divided by the number of observations in the corresponding bin. Then, the 
#' uniform discrimination statistics are calculated via the direct weighted 
#' trapezoidal estimation method such that, for each threshold, the weighted 
#' true positive and false positive rates are cumulatively updated by summing 
#' the weights of the presences and absences, respectively, with that score 
#' (Jiménez-Valverde 2025). The calculation of the uniform statistics requires
#' the classification rule (\code{mat[,1]}) to range between 0 and 1, and the 
#' value of \code{by} to divide 1 exactly. If any of this conditions are not 
#' match, the function stops. A warning message is displayed if (1) the sample 
#' size is lower than 30, (2) any bin has a sample size of zero, or (3) any bin 
#' has a sample size between 1 and 15. In the latter case, trimming should be 
#' considered using \code{deleteBins}, in which case the uniform statistics are 
#' computed excluding the selected bins. See Jiménez-Valverde (2022) for further 
#' details. 
#' 
#' Alternatively, users may wish to downweight the importance of certain 
#' observations relative to others for reasons unrelated to the 
#' representativeness effect (Jiménez-Valverde 2025). For this purpose, the 
#' weights associated to each case can be fully customized with the \code{w} 
#' parameter (see Examples). The length of the weight vector has to be equal to 
#' \code{dim(mat)[1]}.       
#' 
#' The standard \emph{AUC} (non-uniform, unweighted) is estimated 
#' non-parametrically by the trapezoidal rule, which is equivalent to the 
#' Wilcoxon-based estimation (Hanley & McNeil 1982) used in \code{AUCuniform}. 
#' \emph{Se*} is calculated as in \code{AUCuniform}.
#' @return A list with the following elements:
#' @return \code{AUC}: the standard \emph{AUC} value (unweighted), a numeric 
#' value between 0 and 1.
#' @return \code{Se}: the standard \emph{Se*} value (unweighted), a numeric 
#' value between 0 and 1.
#' @return \code{bins}: a table with the sample size of each bin (returned only 
#' if \code{w = NULL}).
#' @return \code{uAUC}: the uniform \emph{AUC} value (returned only if 
#' \code{w = NULL}).
#' @return \code{uSe}: the uniform \emph{Se*} value (returned only if 
#' \code{w = NULL}).
#' @return \code{wAUC}: the weighted \emph{AUC} estimated with the vector
#' \code{w} (returned only if \code{w} is not \code{NULL}).
#' @return \code{wSe}: the weighted \emph{Se*} estimated with the vector
#' \code{w} (returned only if \code{w} is not \code{NULL}).
#' @return \code{TP}: a vector with the true positive rate for every threshold.
#' @return \code{FP}: a vector with the false positive rate for every threshold.
#' @return \code{TP.W}: a vector with the weighted true positive rate for every
#' threshold.
#' @return \code{FP.W}: a vector with the weighted false positive rate for every
#' threshold.
#' @examples
#' # In this first example, a data set is simulated in such a way that the
#' # classification rule is well-calibrated, i.e., the observed proportion of 
#' # positive cases equates to the simulated probabilities of presence. Since
#' # the objective is to calculate the uAUC to account for the environmental 
#' # representativeness effect (see Jiménez-Valverde 2022), weights are 
#' # automatically calculated and no w vector is needed. 
#' 
#' n <- 1000 # Set the sample size
#' hs <- rbeta(n, 2, 2) # Simulated probabilities (the classification rule)
#' random <- runif(n)
#' sp <- ifelse(random < hs, 1, 0) # Observed presence–absence data
#' 
#' result <- AUCuniform.2(cbind(hs, sp), plot = TRUE, plot.compare = TRUE)
#' 
#' result$AUC  # Get the standard AUC
#' result$uAUC # Get the uniform AUC. Note how it is close to the reference value
#'             # of 0.83 since the probability values  (the classification rule)
#'             # are simulated to be well-calibrated (see Jiménez-Valverde 2022)
#' 
#' # In this second set of examples, the objective is not to calculate the  
#' # uniform AUC, but to assign specific weights to certain observations. These 
#' # examples corresponds to some of those provided in Table 1 of 
#' # Jiménez-Valverde (2025).
#' 
#' hs <- seq(1, 0.05, by = -0.05) # Generate the classification rule 
#' sp <- c(0, 1, 1, 0, 1, 1, 1, 1, 1, 0, 1, 0, 0, 0, 1, 1, 1, 0, 0, 0) # Observed presence–absence data
#' 
#' wa <- ifelse(sp == 0, 0.2, 1) # The vector of weights for each case
#' result.a <- AUCuniform.2(cbind(hs, sp), w = wa, plot = TRUE, plot.compare = TRUE)
#' 
#' result.a$AUC  # Get the standard AUC
#' result.a$wAUC # Get the weighted AUC. Since every case within each category of
#'               # sp received the same weight, the weighted AUC value equals the
#'               # standard AUC value
#'
#' wb <- c(rep(1, 19), 0.2) # The vector of weights for each case
#' result.b <- AUCuniform.2(cbind(hs, sp), w = wb, plot = TRUE, plot.compare = TRUE)
#' 
#' result.b$wAUC # Get the weighted AUC. Since a low weight is assigned to an 
#'               # instance of absence associated with a low probability value, 
#'               # the weighted AUC is lower than the standard AUC value.
#'             
#' wc <- c(0.2, rep(1, 19)) # The vector of weights for each case
#' result.c <- AUCuniform.2(cbind(hs, sp), w = wc, plot = TRUE, plot.compare = TRUE)
#' 
#' result.c$wAUC # Get the weighted AUC. Since a low weight is assigned to an 
#'               # instance of absence associated with a high probability value, 
#'               # the weighted AUC is higher than the standard AUC value
#' @encoding UTF-8
#' @references Hanley, J. A. & McNeil, B. J. (1982). The Meaning and Use of the
#'     Area under a Receiver Operating Characteristic (ROC) Curve.
#'     \emph{Radiology}., 143, 29-36.
#'
#'     Jiménez-Valverde, A. (2022). The uniform AUC: dealing with the
#'     representativeness effect in presence-absence models. \emph{Methods Ecol.
#'     Evol.}, 13, 1224-1236.
#'
#'     Jiménez-Valverde, A. (2025). Refining uniform discrimination metrics: 
#'     towards a case-by-case weighting evaluation in species distribution 
#'     models with presence-absence data. \emph{Ecol. Evol.}, 15, e72573. 
#' @importFrom graphics plot
#' @importFrom graphics abline
#' @importFrom graphics axis
#' @importFrom graphics points
#' @importFrom utils head tail
#' @export

AUCuniform.2 <- function(mat, by = 0.1, deleteBins = NULL, w = NULL, plot = FALSE, plot.compare = FALSE, plot.adds = FALSE) {

    if (!plot) {
        plot.compare <- FALSE
        plot.adds <- FALSE
    } else if (!plot.compare) {
        plot.adds <- FALSE
    }

    mat.ord <- mat[order(mat[, 1], decreasing = TRUE), ]

    unique.th <- sort(unique(mat.ord[, 1]), decreasing = TRUE)
    true.pos <- c()
    false.pos <- c()
    n.pos <- sum(mat.ord[, 2])
    n.neg <- length(mat.ord[, 2]) - n.pos
    if (n.pos == 0 || n.neg == 0) {
        stop("The dataset must contain both positive and negative cases.")
    }
    tp <- 0
    fp <- 0
    for (th in unique.th) {
        group <- mat.ord[mat.ord[, 1] == th, , drop = FALSE]
        tp <- tp + sum(group[, 2] == 1)
        fp <- fp + sum(group[, 2] == 0)
        true.pos <- c(true.pos, tp/n.pos)
        false.pos <- c(false.pos, fp/n.neg)
    }
    true.pos <- c(0, true.pos)
    false.pos <- c(0, false.pos)
    auc.trap <- sum(diff(false.pos) * (head(true.pos, -1) + tail(true.pos, -1))/2)
    diferencia <- abs(true.pos - (1 - false.pos))
    se.trap <- (true.pos[which.min(diferencia)] + (1 - false.pos)[which.min(diferencia)])/2

    if (is.null(w) == FALSE) {
        if (length(w) != dim(mat)[1])
            stop("The number of cases does not match the length of w.")
        probs.ord <- w[order(mat[, 1], decreasing = TRUE)]
        mat.ord <- cbind(mat.ord, probs.ord)
        unique.th <- sort(unique(mat.ord[, 1]), decreasing = TRUE)
        true.pos.w <- c()
        false.pos.w <- c()
        n.pos.w <- sum(mat.ord[mat.ord[, 2] == 1, 3])
        n.neg.w <- sum(mat.ord[mat.ord[, 2] == 0, 3])
        tp.w <- 0
        fp.w <- 0
        for (th in unique.th) {
            group <- mat.ord[mat.ord[, 1] == th, , drop = FALSE]
            tp.w <- tp.w + sum(group[group[, 2] == 1, 3])
            fp.w <- fp.w + sum(group[group[, 2] == 0, 3])
            true.pos.w <- c(true.pos.w, tp.w/n.pos.w)
            false.pos.w <- c(false.pos.w, fp.w/n.neg.w)
        }
        true.pos.w <- c(0, true.pos.w)
        false.pos.w <- c(0, false.pos.w)
        w.auc <- sum(diff(false.pos.w) * (head(true.pos.w, -1) + tail(true.pos.w, -1))/2)
        diferencia.w <- abs(true.pos.w - (1 - false.pos.w))
        w.se <- (true.pos.w[which.min(diferencia.w)] + (1 - false.pos.w)[which.min(diferencia.w)])/2
    }

    if (is.null(w) == TRUE) {
        if (any(mat[, 1] < 0 | mat[, 1] > 1)) {
            stop("The classification rule is out of the range [0,1]; consider rescaling (min-max normalization) before proceeding.")
        }
        if (abs((1/by) - round(1/by)) > .Machine$double.eps^0.5) {
            stop("'by' must divide 1 exactly (e.g., 0.2, 0.1, 0.05).")
        }
        if (dim(mat)[1] < 30)
            warning("Your sample size is low, results must be interpreted with caution.")
        bins <- seq(0, 1, by)
        intervals <- cut(mat[, 1], bins, include.lowest = TRUE, right = FALSE)
        if (sum(table(intervals) == 0) > 0)
            warning(paste("There are (is)", sum(table(intervals) == 0), "interval(s) with zero data, results must be interpreted with caution."))
        if (sum(table(intervals) < 15) > 0)
            warning(paste("There are (is)", sum(table(intervals) < 15) - sum(table(intervals) == 0), "interval(s) with 0<n<15 data, results must be interpreted with caution."))
        probs <- as.vector(1/table(intervals)[intervals])
        probs.ord <- probs[order(mat[, 1], decreasing = TRUE)]
        if (is.null(deleteBins) == FALSE) {
            toDelete <- levels(intervals)[deleteBins]
            mat <- mat[(intervals %in% toDelete) == FALSE, ]
            probs <- probs[(intervals %in% toDelete) == FALSE]
            mat.ord <- mat[order(mat[, 1], decreasing = TRUE), ]
            probs.ord <- probs[order(mat[, 1], decreasing = TRUE)]
        }
        mat.ord <- cbind(mat.ord, probs.ord)
        unique.th <- sort(unique(mat.ord[, 1]), decreasing = TRUE)
        true.pos.w <- c()
        false.pos.w <- c()
        n.pos.w <- sum(mat.ord[mat.ord[, 2] == 1, 3])
        n.neg.w <- sum(mat.ord[mat.ord[, 2] == 0, 3])
        tp.w <- 0
        fp.w <- 0
        for (th in unique.th) {
            group <- mat.ord[mat.ord[, 1] == th, , drop = FALSE]
            tp.w <- tp.w + sum(group[group[, 2] == 1, 3])
            fp.w <- fp.w + sum(group[group[, 2] == 0, 3])
            true.pos.w <- c(true.pos.w, tp.w/n.pos.w)
            false.pos.w <- c(false.pos.w, fp.w/n.neg.w)
        }
        true.pos.w <- c(0, true.pos.w)
        false.pos.w <- c(0, false.pos.w)
        w.auc <- sum(diff(false.pos.w) * (head(true.pos.w, -1) + tail(true.pos.w, -1))/2)
        diferencia.w <- abs(true.pos.w - (1 - false.pos.w))
        w.se <- (true.pos.w[which.min(diferencia.w)] + (1 - false.pos.w)[which.min(diferencia.w)])/2
    }

    if (plot == TRUE) {
        plot(false.pos, true.pos, pch = 16, xlab = "false positive rate", ylab = "sensitivity", main = "ROC curve",
            yaxt = "n", cex.lab = 1.3, cex.axis = 1, col = "gray")
        axis(side = 2, las = 2, mgp = c(3, 0.75, 0))
        abline(a = 0, b = 1, lty = 2)
    }
    if (plot.compare == TRUE) {
        lines(c(false.pos.w, 1), c(true.pos.w, 1), pch = 16, col = "black")
    }
    if (plot.adds == TRUE) {
        abline(a = 1, b = -1, col = "darkgrey", lty = 2)
        points(1 - se.trap, se.trap, col = "blue", pch = 16)
        points(1 - w.se, w.se, col = "red", pch = 16)
    }
    if (is.null(w) == TRUE) {
        return(list(AUC = auc.trap, Se = se.trap, uAUC = w.auc, uSe = w.se, bins = table(intervals), TP = true.pos,
            FP = false.pos, TP.W = true.pos.w, FP.W = false.pos.w))
    }
    if (is.null(w) == FALSE) {
        return(list(AUC = auc.trap, Se = se.trap, wAUC = w.auc, wSe = w.se, TP = true.pos, FP = false.pos, TP.W = true.pos.w,
            FP.W = false.pos.w))
    }
}

#' @title Calculates the uniform \emph{AUC} and uniform \emph{Se*}
#'
#' @description This function computes the uniform \emph{AUC} (\emph{uAUC}) and
#' uniform \emph{Se*} (\emph{uSe*}) following Jiménez-Valverde (2022). A revised 
#' and improved formulation is available in \code{\link{AUCuniform.2}}, which 
#' users are encouraged to consider for most applications. \code{\link{AUCuniform}}
#' is retained for completeness and reproducibility.
#' @param mat A matrix with two columns. The first column must contain the
#' suitability values (i.e., the classification rule); the second column must
#' contain the presences and absences.
#' @param rep Number of sampling replications. By default, \code{rep} = 100.
#' @param by Size of the suitability intervals (i.e., bins). By default,
#' \code{by} = 0.1.
#' @param deleteBins A vector (e.g., from 1 to 10 if \code{by} = 0.1) with the
#' bins that have to be excluded (1 for [0,0.1), 10 for [0.9,1]) from the
#' resampling procedure (trimming); \code{NULL} by default.
#' @param plot Logical. Indicates whether or not the observed ROC curve is
#' plotted.
#' @param plot.adds Logical. Indicates whether or not the negative diagonal and
#' the point of equivalence are added to the observed ROC plot.
#' @details This function performs the stratified weighted bootstrap to
#' calculate the uniform \emph{AUC} (\emph{uAUC}) and uniform \emph{Se*}
#' (\emph{uSe*}) as suggested in Jiménez-Valverde (2022). A warning message
#' will be shown if the sample size of any bin is zero. Another warning message
#' will be shown if the sample size of any bin is lower than 15. In such case,
#' trimming should be considered. The \emph{AUC} (non-uniform) is estimated
#' non-parametrically (Bamber 1975). \emph{Se*} is calculated by selecting the
#' point that minimizes the absolute difference between sensitivity and
#' specificity and by doing the mean of those values (Jiménez-Valverde 2020).
#' @return A list with the following elements:
#' @return \code{AUC}: the \emph{AUC} value (non-uniform), a numeric value 
#' between 0 and 1.
#' @return \code{Se}: the \emph{Se*} value (non-uniform), a numeric value 
#' between 0 and 1. 
#' @return \code{bins}: a table with the sample size of each bin.
#' @return \code{suit.sim}: a matrix with the bootstrapped suitability values.
#' @return \code{sp.sim}: a matrix with the bootstrapped presence-absence data.
#' @return \code{uAUC}: a numeric vector with the (\emph{uAUC}) values for each
#' replication.
#' @return \code{uAUC.95CI}: a numeric vector with the sample (\emph{uAUC}) 
#' quantiles corresponding to the probabilities 0.025, 0.5 and 0.975.
#' @return \code{uSe}: a numeric vector with the (\emph{uSe*}) values for each
#' replication.
#' @return \code{uSe.95CI}: a numeric vector with the sample (\emph{uSe*}) 
#' quantiles corresponding to the probabilities 0.025, 0.5 and 0.975.
#' @examples
#' suit<-rbeta(100, 2, 2) # Generate suitability values
#' random<-runif(100)
#' sp<-ifelse(random < suit, 1, 0) # Generate presence-absence data
#' result<-AUCuniform(cbind(suit, sp), plot = TRUE, plot.adds = TRUE)
#' result$uAUC.95CI[2] # Get the uAUC
#' @encoding UTF-8
#' @references Bamber, D. (1975). The Area above the Ordinal Dominance Graph and
#'     the Area below the Receiver Operating Characteristic Graph.
#'     \emph{J. Math. Psychol}., 12, 387-415.
#'
#'     Jiménez-Valverde, A. (2020). Sample size for the evaluation of
#'     presence-absence models. \emph{Ecol. Indic}., 114, 106289.
#'
#'     Jiménez-Valverde, A. (2022). The uniform AUC: dealing with the
#'     representativeness effect in presence-absence models. \emph{Methods Ecol.
#'     Evol.}, 13, 1224-1236.
#' @importFrom stats quantile wilcox.test
#' @importFrom graphics plot
#' @importFrom graphics abline
#' @importFrom graphics axis
#' @importFrom graphics points
#' @export

AUCuniform <- function(mat, rep = 100, by = 0.1, deleteBins = NULL, plot = FALSE, plot.adds = FALSE) {
    # Non-uniform:
    Wilcox <- wilcox.test(mat[, 1] ~ mat[, 2], exact = FALSE)
    AUC <- 1 - (Wilcox["statistic"]$statistic/((length(mat[, 2]) - (sum(mat[, 2]))) * sum(mat[, 2])))
    cosa <- ROCR::prediction(mat[, 1], mat[, 2])
    temporal <- ROCR::performance(cosa, "sens", "spec")
    diferencia <- abs(temporal@x.values[[1]] - temporal@y.values[[1]])
    SE <- (temporal@y.values[[1]][which.min(diferencia)] + temporal@x.values[[1]][which.min(diferencia)])/2
    if (plot == TRUE) {
        plot(1 - temporal@x.values[[1]], temporal@y.values[[1]], pch = 16, xlab = "false positive rate", ylab = "sensitivity",
            main = "ROC curve", yaxt = "n", cex.lab = 1.3, cex.axis = 1)
        axis(side = 2, las = 2, mgp = c(3, 0.75, 0))
        abline(a = 0, b = 1, lty = 2)
        if (plot.adds == TRUE) {
            abline(a = 1, b = -1, col = "darkgrey", lty = 2)
            points(1 - SE, SE, col = "red", pch = 16)
        }
    }
    # Uniform:
    bins <- seq(0, 1, by)
    intervals <- cut(mat[, 1], bins, include.lowest = T, right = F)
    probs <- table(intervals)[intervals]
    if (dim(mat)[1] < 30)
        warning("Your sample size is low, results must be interpreted with caution.")
    if (sum(table(intervals) < 15) > 0)
        warning("At least one sutability interval has n < 15, results must be interpreted with caution.")
    if (sum(table(intervals) == 0) > 0)
        warning(paste("There are", sum(table(intervals) == 0), "interval(s) with zero data, results must be interpreted with caution."))
    if (is.null(deleteBins) == FALSE) {
        toDelete <- levels(intervals)[deleteBins]
        mat <- mat[(intervals %in% toDelete) == FALSE, ]
        probs <- probs[(intervals %in% toDelete) == FALSE]
    }
    uAUC <- c()
    uSE <- c()
    HS <- matrix(nrow = nrow(mat), ncol = rep)
    SP <- matrix(nrow = nrow(mat), ncol = rep)
    for (i in 1:rep) {
        newdata <- mat[sample(nrow(mat), nrow(mat), replace = T, prob = 1/probs), ]
        HS[, i] <- newdata[, 1]
        SP[, i] <- newdata[, 2]
        Wilcox <- wilcox.test(newdata[, 1] ~ newdata[, 2], exact = FALSE)
        uAUC <- c(uAUC, 1 - (Wilcox["statistic"]$statistic/((dim(newdata)[1] - (sum(newdata[, 2]))) * sum(newdata[,
            2])))[[1]])
        cosa <- ROCR::prediction(newdata[, 1], newdata[, 2])
        temporal <- ROCR::performance(cosa, "sens", "spec")
        diferencia <- abs(temporal@x.values[[1]] - temporal@y.values[[1]])
        uSE <- c(uSE, (temporal@y.values[[1]][which.min(diferencia)] + temporal@x.values[[1]][which.min(diferencia)])/2)
    }

    return(list(AUC = AUC[[1]], Se = SE, bins = table(intervals), suit.sim = HS, sp.sim = SP, uAUC = uAUC, uAUC.95CI = quantile(uAUC,
        c(0.025, 0.5, 0.975)), uSe = uSE, uSe.95CI = quantile(uSE, c(0.025, 0.5, 0.975))))
}

#' @title Suitability values distribution graph
#'
#' @description A function to visualize the distribution of the suitability
#' values associated to presences, absences, and all cases together.
#' @param mat A matrix with two columns. The first column must contain the
#' suitability values (i.e., the classification rule); the second column must
#' contain the presences and absences.
#' @param breaks Number of cells for the total histogram. By default,
#' \code{breaks} = 10.
#' @param hist.total Logical. Indicates whether or not the distribution of
#' suitability values for all the cases together is graphed.
#' @details In blue, the distribution of the suitability values associated to
#' presences. In red, the distribution of the suitability values associated to
#' absences. This graph helps to understand why the \emph{AUC} (or \emph{Se*})
#' is greater, equal to, or less than the \emph{uAUC} (or \emph{uSe*}) (see
#' Jiménez-Valverde 2022).
#' @return This function returns a multiple histogram.
#' @examples
#' suit<-rbeta(100, 2, 2) # Generate suitability values
#' random<-runif(100)
#' sp<-ifelse(random < suit, 1 , 0) # Generate presence-absence data
#' HSgraph(cbind(suit, sp), breaks = 20, hist.total = TRUE)
#' @encoding UTF-8
#' @references Jiménez-Valverde, A. (2022). The uniform AUC: dealing with the
#'     representativeness effect in presence-absence models. \emph{Methods Ecol.
#'     Evol.}, 13, 1224-1236.
#' @importFrom graphics hist
#' @importFrom grDevices rgb
#' @importFrom graphics lines
#' @export

HSgraph <- function(mat, breaks = 10, hist.total = TRUE) {
    breaks <- seq(min(mat[, 1]), max(mat[, 1]), length.out = breaks + 1)
    histpresence <- hist(mat[, 1][mat[, 2] == 1], breaks = breaks)
    histabsence <- hist(mat[, 1][mat[, 2] == 0], breaks = breaks)
    histtotal <- hist(mat[, 1], breaks = breaks)
    plot(histpresence, col = rgb(0, 0, 1, 1/4), xlim = c(min(mat[, 1]), max(mat[, 1])), ylim = c(0, max(histpresence$counts,
        histabsence$counts, histtotal$counts)), border = rgb(0, 0, 1, 0), main = "", xlab = "Suitability", ylab = "Frequency",
        las = 1)
    plot(histabsence, col = rgb(1, 0, 0, 1/4), xlim = c(min(mat[, 1]), max(mat[, 1])), border = rgb(1, 0, 0, 0),
        add = T)
    if (hist.total == TRUE) {
        lines(c(0, histtotal$breaks), c(0, histtotal$counts, 0), type = "s", lwd = 2)
    }
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AUCuniform_trap.R
\encoding{UTF-8}
\name{AUCuniform_trap}
\alias{AUCuniform_trap}
\title{Calculates the uniform \emph{AUC} and uniform \emph{Se*} by the
trapezoidal method.}
\usage{
AUCuniform_trap(
  mat,
  by = 0.1,
  deleteBins = NULL,
  w = NULL,
  plot = FALSE,
  plot.compare = FALSE,
  plot.adds = FALSE
)
}
\arguments{
\item{mat}{A matrix with two columns. The first column must contain the
suitability values (i.e., the classification rule); the second column must
contain the presences and absences.}

\item{by}{Size of the suitability intervals (i.e., bins). By default,
\code{by} = 0.1.}

\item{deleteBins}{A vector (e.g., from 1 to 10 if \code{by} = 0.1) with the
bins that have to be excluded (1 for [0,0.1), 10 for [0.9,1]) from the
calculation of the uniform statistics; \code{NULL} by default.}

\item{w}{A vector with the weights associated to each case. If \code{NULL}
(default), then the uniform \emph{AUC} (\emph{uAUC}) and uniform \emph{Se*}
(\emph{uSe*}) are calculated.}

\item{plot}{Logical. Indicates whether or not the observed ROC curve is
plotted (gray dots).}

\item{plot.compare}{Logical. Indicates whether or not the weighed ROC curve
is plotted (black line).}

\item{plot.adds}{Logical. Indicates whether or not the negative diagonal and
the points of equivalence (weighted and unweighted) are added to the ROC
plot.}
}
\value{
A list with the following elements:

\code{AUC}: the \emph{AUC} value (non-uniform, unweighted), a numeric
value between 0 and 1.

\code{Se}: the \emph{Se*} value (non-uniform, unweighted), a numeric
value between 0 and 1.

\code{bins}: a table with the sample size of each bin (only if
\code{w = NULL}).

\code{uAUC}: the uniform \emph{AUC} value (only if \code{w = NULL}).

\code{uSe}: the uniform \emph{Se*} value (only if \code{w = NULL}).

\code{wAUC}: the weighted \emph{AUC} estimated with the vector
\code{w}

\code{wSe}: the weighted \emph{Se*} estimated with the vector
\code{w}
}
\description{
This function computes the uniform \emph{AUC} (\emph{uAUC}) and
uniform \emph{Se*} (\emph{uSe*}) using the weighted trapezoidal method
instead of the weighted bootstrapping method used in \code{AUCuniform} and
originally proposed in Jiménez-Valverde (2022). This procedure reduces bias 
and improves the coverage of confidence intervals (Jiménez-Valverde 2024). 
Additionally, the weights vector associated to each case can be customized. 
See Jiménez-Valverde (2024) for details.
}
\details{
This function calculates the uniform \emph{AUC} (\emph{uAUC}) and
uniform \emph{Se*} (\emph{uSe*}) using the weighted trapezoidal method as
suggested in Jiménez-Valverde (2024). A warning message will be shown if
the sample size of any bin is zero. Another warning message will be shown if
the sample size of any bin is lower than 15. In such case, trimming should be
considered using \code{deleteBins} (Jiménez-Valverde 2022). Alternatively,
the weights associated to each case can be fully customized with the \code{w}
parameter (Jiménez-Valverde 2024). In this case, no warnings regarding
sample size issues are raised, and \code{deleteBins} is not used. The
\emph{AUC} (non-uniform, unweighted) is estimated non-parametrically by the
trapezoidal rule, which is equivalent to the Wilcoxon-based estimation 
(Hanley & McNeil 1982) used in \code{AUCuniform}. \emph{Se*} is calculated as 
in \code{AUCuniform}.
}
\examples{
\donttest{
suit<-rbeta(100, 2, 2) #Generate suitability values
random<-runif(100)
sp<-ifelse(random < suit, 1, 0) #Generate presence-absence data
result<-AUCuniform_trap(cbind(suit, sp), plot = TRUE, plot.compare = TRUE)
result$AUC #Get the AUC
result$uAUC #Get the uAUC. Note how it is closer to the reference value of
            #0.83 since the suitability values are simulated to be
            #well-calibrated (see Jimenez-Valverde 2022).
            }
}
\references{
Hanley, J. A. & McNeil, B. J. (1982). The Meaning and Use of the
    Area under a Receiver Operating Characteristic (ROC) Curve.
    \emph{Radiology}., 143, 29-36.

    Jiménez-Valverde, A. (2022). The uniform AUC: dealing with the
    representativeness effect in presence-absence models. \emph{Methods Ecol.
    Evol.}, 13, 1224-1236.

    Jiménez-Valverde, A. (2024). Improving the uniform AUC (uAUC): towards a 
    case-by-case weighting evaluation in species distribution models. 
    \emph{In preparation}.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CALplot.R
\encoding{UTF-8}
\name{CALplot}
\alias{CALplot}
\title{Calibration graph}
\usage{
CALplot(mat, by = 0.1)
}
\arguments{
\item{mat}{A matrix with two columns. The first column must contain the
suitability values (i.e., the classification rule); the second column must
contain the presences and absences.}

\item{by}{Size of the suitability intervals (bins). By default,
\code{by} = 0.1.}
}
\value{
This function returns a calibration plot
}
\description{
A function to plot a calibration graph.
}
\details{
Dots for bins with 15 or more cases are shown in solid black; dots
for bins with less than 15 cases are shown empty (see Jiménez-Valverde et
al. 2013). This way, by plotting the calibration graph before running
\code{\link{AUCuniform}}, one can get a glimpse of how reliable \emph{uAUC}
or \emph{uSe*} can be expected to be.
}
\examples{
suit<-rbeta(100, 2, 2) # Generate suitability values
random<-runif(100)
sp<-ifelse(random < suit,1 , 0) # Generate presence-absence data
CALplot(cbind(suit, sp))
}
\references{
Jiménez-Valverde, A., Acevedo, P., Barbosa, A. M., Lobo, J. M. &
    Real, R. (2013).  Discrimination capacity in species distribution models
    depends on the representativeness of the environmental domain.
    \emph{Global Ecol. Biogeogr.}, 22, 508-516.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AUCuniform.2.R
\encoding{UTF-8}
\name{AUCuniform.2}
\alias{AUCuniform.2}
\title{Calculates the uniform \emph{AUC} and uniform \emph{Se*} by the
direct weighted trapezoidal estimation method.}
\usage{
AUCuniform.2(
  mat,
  by = 0.1,
  deleteBins = NULL,
  w = NULL,
  plot = FALSE,
  plot.compare = FALSE,
  plot.adds = FALSE
)
}
\arguments{
\item{mat}{A matrix with two columns. The first column must contain the 
classification rule (e.g., the suitability values); the second column 
must contain the presences and absences.}

\item{by}{The size of the intervals used to divide the classification rule 
(i.e., bins width). By default, \code{by} = 0.1. This argument is only used 
when \code{w = NULL}.}

\item{deleteBins}{A vector (e.g., from 1 to 10 if \code{by} = 0.1) with the
bins that have to be excluded (1 for [0,0.1), 10 for [0.9,1]) from the
calculation of the uniform statistics. The default is \code{NULL}. This 
argument is only used when \code{w = NULL}.}

\item{w}{A vector with the weights associated with each case. If \code{NULL}
(default), each case is weighted by the inverse of the sample size of 
its corresponding bin, and the uniform \emph{AUC} (\emph{uAUC}) and uniform 
\emph{Se*} (\emph{uSe*}) are calculated (see Details).}

\item{plot}{Logical. If \code{TRUE}, the standard (unweighted) ROC curve is 
plotted (gray dots).}

\item{plot.compare}{Logical. If \code{TRUE}, the weighed ROC curve is plotted
(black line).}

\item{plot.adds}{Logical. If \code{TRUE}, adds the negative diagonal and the
points of equivalence (weighted and unweighted) to the ROC plot.}
}
\value{
A list with the following elements:

\code{AUC}: the standard \emph{AUC} value (unweighted), a numeric 
value between 0 and 1.

\code{Se}: the standard \emph{Se*} value (unweighted), a numeric 
value between 0 and 1.

\code{bins}: a table with the sample size of each bin (returned only 
if \code{w = NULL}).

\code{uAUC}: the uniform \emph{AUC} value (returned only if 
\code{w = NULL}).

\code{uSe}: the uniform \emph{Se*} value (returned only if 
\code{w = NULL}).

\code{wAUC}: the weighted \emph{AUC} estimated with the vector
\code{w} (returned only if \code{w} is not \code{NULL}).

\code{wSe}: the weighted \emph{Se*} estimated with the vector
\code{w} (returned only if \code{w} is not \code{NULL}).

\code{TP}: a vector with the true positive rate for every threshold.

\code{FP}: a vector with the false positive rate for every threshold.

\code{TP.W}: a vector with the weighted true positive rate for every
threshold.

\code{FP.W}: a vector with the weighted false positive rate for every
threshold.
}
\description{
This function computes the uniform \emph{AUC} (\emph{uAUC}) and
uniform \emph{Se*} (\emph{uSe*}) using the direct weighted trapezoidal
estimation method (Jiménez-Valverde 2025), instead of the stratified 
bootstrapping with inverse probability weighting method implemented in 
\code{\link{AUCuniform}} and originally proposed by Jiménez-Valverde (2022). 
Uniform statistics are design to account for the representativeness effect 
(Jiménez-Valverde 2022). This new method reduces bias and improves the 
coverage of confidence intervals relative to the original proposal. 
Additionally, the weight vector associated to each case can be customized.
}
\details{
This function calculates the uniform \emph{AUC} (\emph{uAUC}) and
uniform \emph{Se*} (\emph{uSe*}) using the direct weighted trapezoidal 
estimation method proposed by Jiménez-Valverde (2025). To compute the uniform
statistics, the \emph{w} parameter must be set to \emph{NULL} (default). The 
data set is divided into bins (defined by the parameter \code{by}) based on 
the values of the first vector in the input matrix \code{mat} (the 
classification rule). Each observation is assigned a weight equal to one 
divided by the number of observations in the corresponding bin. Then, the 
uniform discrimination statistics are calculated via the direct weighted 
trapezoidal estimation method such that, for each threshold, the weighted 
true positive and false positive rates are cumulatively updated by summing 
the weights of the presences and absences, respectively, with that score 
(Jiménez-Valverde 2025). The calculation of the uniform statistics requires
the classification rule (\code{mat[,1]}) to range between 0 and 1, and the 
value of \code{by} to divide 1 exactly. If any of this conditions are not 
match, the function stops. A warning message is displayed if (1) the sample 
size is lower than 30, (2) any bin has a sample size of zero, or (3) any bin 
has a sample size between 1 and 15. In the latter case, trimming should be 
considered using \code{deleteBins}, in which case the uniform statistics are 
computed excluding the selected bins. See Jiménez-Valverde (2022) for further 
details. 

Alternatively, users may wish to downweight the importance of certain 
observations relative to others for reasons unrelated to the 
representativeness effect (Jiménez-Valverde 2025). For this purpose, the 
weights associated to each case can be fully customized with the \code{w} 
parameter (see Examples). The length of the weight vector has to be equal to 
\code{dim(mat)[1]}.       

The standard \emph{AUC} (non-uniform, unweighted) is estimated 
non-parametrically by the trapezoidal rule, which is equivalent to the 
Wilcoxon-based estimation (Hanley & McNeil 1982) used in \code{AUCuniform}. 
\emph{Se*} is calculated as in \code{AUCuniform}.
}
\examples{
# In this first example, a data set is simulated in such a way that the
# classification rule is well-calibrated, i.e., the observed proportion of 
# positive cases equates to the simulated probabilities of presence. Since
# the objective is to calculate the uAUC to account for the environmental 
# representativeness effect (see Jiménez-Valverde 2022), weights are 
# automatically calculated and no w vector is needed. 

n <- 1000 # Set the sample size
hs <- rbeta(n, 2, 2) # Simulated probabilities (the classification rule)
random <- runif(n)
sp <- ifelse(random < hs, 1, 0) # Observed presence–absence data

result <- AUCuniform.2(cbind(hs, sp), plot = TRUE, plot.compare = TRUE)

result$AUC  # Get the standard AUC
result$uAUC # Get the uniform AUC. Note how it is close to the reference value
            # of 0.83 since the probability values  (the classification rule)
            # are simulated to be well-calibrated (see Jiménez-Valverde 2022)

# In this second set of examples, the objective is not to calculate the  
# uniform AUC, but to assign specific weights to certain observations. These 
# examples corresponds to some of those provided in Table 1 of 
# Jiménez-Valverde (2025).

hs <- seq(1, 0.05, by = -0.05) # Generate the classification rule 
sp <- c(0, 1, 1, 0, 1, 1, 1, 1, 1, 0, 1, 0, 0, 0, 1, 1, 1, 0, 0, 0) # Observed presence–absence data

wa <- ifelse(sp == 0, 0.2, 1) # The vector of weights for each case
result.a <- AUCuniform.2(cbind(hs, sp), w = wa, plot = TRUE, plot.compare = TRUE)

result.a$AUC  # Get the standard AUC
result.a$wAUC # Get the weighted AUC. Since every case within each category of
              # sp received the same weight, the weighted AUC value equals the
              # standard AUC value

wb <- c(rep(1, 19), 0.2) # The vector of weights for each case
result.b <- AUCuniform.2(cbind(hs, sp), w = wb, plot = TRUE, plot.compare = TRUE)

result.b$wAUC # Get the weighted AUC. Since a low weight is assigned to an 
              # instance of absence associated with a low probability value, 
              # the weighted AUC is lower than the standard AUC value.
            
wc <- c(0.2, rep(1, 19)) # The vector of weights for each case
result.c <- AUCuniform.2(cbind(hs, sp), w = wc, plot = TRUE, plot.compare = TRUE)

result.c$wAUC # Get the weighted AUC. Since a low weight is assigned to an 
              # instance of absence associated with a high probability value, 
              # the weighted AUC is higher than the standard AUC value
}
\references{
Hanley, J. A. & McNeil, B. J. (1982). The Meaning and Use of the
    Area under a Receiver Operating Characteristic (ROC) Curve.
    \emph{Radiology}., 143, 29-36.

    Jiménez-Valverde, A. (2022). The uniform AUC: dealing with the
    representativeness effect in presence-absence models. \emph{Methods Ecol.
    Evol.}, 13, 1224-1236.

    Jiménez-Valverde, A. (2025). Refining uniform discrimination metrics: 
    towards a case-by-case weighting evaluation in species distribution 
    models with presence-absence data. \emph{Ecol. Evol.}, 15, e72573.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AUCuniform.R
\encoding{UTF-8}
\name{AUCuniform}
\alias{AUCuniform}
\title{Calculates the uniform \emph{AUC} and uniform \emph{Se*}}
\usage{
AUCuniform(
  mat,
  rep = 100,
  by = 0.1,
  deleteBins = NULL,
  plot = FALSE,
  plot.adds = FALSE
)
}
\arguments{
\item{mat}{A matrix with two columns. The first column must contain the
suitability values (i.e., the classification rule); the second column must
contain the presences and absences.}

\item{rep}{Number of sampling replications. By default, \code{rep} = 100.}

\item{by}{Size of the suitability intervals (i.e., bins). By default,
\code{by} = 0.1.}

\item{deleteBins}{A vector (e.g., from 1 to 10 if \code{by} = 0.1) with the
bins that have to be excluded (1 for [0,0.1), 10 for [0.9,1]) from the
resampling procedure (trimming); \code{NULL} by default.}

\item{plot}{Logical. Indicates whether or not the observed ROC curve is
plotted.}

\item{plot.adds}{Logical. Indicates whether or not the negative diagonal and
the point of equivalence are added to the observed ROC plot.}
}
\value{
A list with the following elements:

\code{AUC}: the \emph{AUC} value (non-uniform), a numeric value 
between 0 and 1.

\code{Se}: the \emph{Se*} value (non-uniform), a numeric value 
between 0 and 1.

\code{bins}: a table with the sample size of each bin.

\code{suit.sim}: a matrix with the bootstrapped suitability values.

\code{sp.sim}: a matrix with the bootstrapped presence-absence data.

\code{uAUC}: a numeric vector with the (\emph{uAUC}) values for each
replication.

\code{uAUC.95CI}: a numeric vector with the sample (\emph{uAUC}) 
quantiles corresponding to the probabilities 0.025, 0.5 and 0.975.

\code{uSe}: a numeric vector with the (\emph{uSe*}) values for each
replication.

\code{uSe.95CI}: a numeric vector with the sample (\emph{uSe*}) 
quantiles corresponding to the probabilities 0.025, 0.5 and 0.975.
}
\description{
This function computes the uniform \emph{AUC} (\emph{uAUC}) and
uniform \emph{Se*} (\emph{uSe*}) following Jiménez-Valverde (2022). A revised 
and improved formulation is available in \code{\link{AUCuniform.2}}, which 
users are encouraged to consider for most applications. \code{\link{AUCuniform}}
is retained for completeness and reproducibility.
}
\details{
This function performs the stratified weighted bootstrap to
calculate the uniform \emph{AUC} (\emph{uAUC}) and uniform \emph{Se*}
(\emph{uSe*}) as suggested in Jiménez-Valverde (2022). A warning message
will be shown if the sample size of any bin is zero. Another warning message
will be shown if the sample size of any bin is lower than 15. In such case,
trimming should be considered. The \emph{AUC} (non-uniform) is estimated
non-parametrically (Bamber 1975). \emph{Se*} is calculated by selecting the
point that minimizes the absolute difference between sensitivity and
specificity and by doing the mean of those values (Jiménez-Valverde 2020).
}
\examples{
suit<-rbeta(100, 2, 2) # Generate suitability values
random<-runif(100)
sp<-ifelse(random < suit, 1, 0) # Generate presence-absence data
result<-AUCuniform(cbind(suit, sp), plot = TRUE, plot.adds = TRUE)
result$uAUC.95CI[2] # Get the uAUC
}
\references{
Bamber, D. (1975). The Area above the Ordinal Dominance Graph and
    the Area below the Receiver Operating Characteristic Graph.
    \emph{J. Math. Psychol}., 12, 387-415.

    Jiménez-Valverde, A. (2020). Sample size for the evaluation of
    presence-absence models. \emph{Ecol. Indic}., 114, 106289.

    Jiménez-Valverde, A. (2022). The uniform AUC: dealing with the
    representativeness effect in presence-absence models. \emph{Methods Ecol.
    Evol.}, 13, 1224-1236.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/HSgraph.R
\encoding{UTF-8}
\name{HSgraph}
\alias{HSgraph}
\title{Suitability values distribution graph}
\usage{
HSgraph(mat, breaks = 10, hist.total = TRUE)
}
\arguments{
\item{mat}{A matrix with two columns. The first column must contain the
suitability values (i.e., the classification rule); the second column must
contain the presences and absences.}

\item{breaks}{Number of cells for the total histogram. By default,
\code{breaks} = 10.}

\item{hist.total}{Logical. Indicates whether or not the distribution of
suitability values for all the cases together is graphed.}
}
\value{
This function returns a multiple histogram.
}
\description{
A function to visualize the distribution of the suitability
values associated to presences, absences, and all cases together.
}
\details{
In blue, the distribution of the suitability values associated to
presences. In red, the distribution of the suitability values associated to
absences. This graph helps to understand why the \emph{AUC} (or \emph{Se*})
is greater, equal to, or less than the \emph{uAUC} (or \emph{uSe*}) (see
Jiménez-Valverde 2022).
}
\examples{
suit<-rbeta(100, 2, 2) # Generate suitability values
random<-runif(100)
sp<-ifelse(random < suit, 1 , 0) # Generate presence-absence data
HSgraph(cbind(suit, sp), breaks = 20, hist.total = TRUE)
}
\references{
Jiménez-Valverde, A. (2022). The uniform AUC: dealing with the
    representativeness effect in presence-absence models. \emph{Methods Ecol.
    Evol.}, 13, 1224-1236.
}
